from infi.shell import availability, command
from infi.shell.utils import strlist

from ..utils import Version
from .system import system, Roles

def camelize(s):
    return s.title().replace('_', '')

VERSION_2 = Version('2.0.0.0')
def is_pre_v2_system(api=None):
    api = api or system.api
    if not api.system_version:
        api.ping()
        if not api.system_version:
            api._api.system_version = Version(api.get('system/version'))
    return api.system_version < VERSION_2

def pre_v2_patches(api):
    if not is_pre_v2_system(api):
        return
    # Convert user roles to camel case
    for attr in strlist('INFINIDAT ADMIN POOLADMIN TECHNICIAN READONLY'):
        setattr(Roles, attr, camelize(getattr(Roles, attr)))

def v2_and_above(func):
    return availability(func, predicate=lambda: not is_pre_v2_system())

def v2_command(func=None, **kwargs):
    def decorator(_func):
        return v2_and_above(command(_func, **kwargs))
    if func is None:
        return decorator
    return decorator(func)

def below_v2(func):
    return availability(func, predicate=lambda: is_pre_v2_system())

def pre_v2_command(func=None, **kwargs):
    def decorator(_func):
        return below_v2(command(_func, **kwargs))
    if func is None:
        return decorator
    return decorator(func)


